@@ -1,6 +1,6 @@
 ;; geiser-racket.el -- geiser support for Racket scheme
 
-;; Copyright (C) 2009, 2010, 2011, 2012, 2013 Jose Antonio Ortega Ruiz
+;; Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jose Antonio Ortega Ruiz
 
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the Modified BSD License. You should
@@ -61,8 +61,7 @@ This executable is used by `run-gracket', and, if
   :group 'geiser-racket)
 
 (geiser-custom--defcustom geiser-racket-extra-keywords
-    '("define-syntax-rule" "provide" "require"
-      "unless" "when" "with-handlers")
+    '("provide" "require" "unless" "when" "with-handlers")
   "Extra keywords highlighted in Racket buffers."
   :type '(repeat string)
   :group 'geiser-racket)
@@ -120,9 +119,7 @@ using start-geiser, a procedure in the geiser/server module."
 
 (defun geiser-racket--explicit-module ()
   (save-excursion
-    (ignore-errors
-      (while (not (zerop (geiser-syntax--nesting-level)))
-        (backward-up-list)))
+    (geiser-syntax--pop-to-top)
     (and (looking-at geiser-racket--module-re)
          (let ((mod (match-string-no-properties 1))
                (lang (match-string-no-properties 2)))
@@ -137,11 +134,14 @@ using start-geiser, a procedure in the geiser/server module."
             (geiser-syntax--form-from-string (match-string-no-properties 1))))
       "#f"))
 
-(defun geiser-racket--implicit-module ()
+(defun geiser-racket--implicit-module (&optional pos)
   (save-excursion
     (goto-char (point-min))
     (when (re-search-forward "^#lang " nil t)
-      (buffer-file-name))))
+      (if pos (progn (end-of-line) (list (point))) (buffer-file-name)))))
+
+(defun geiser-racket--eval-bounds ()
+  (geiser-racket--implicit-module t))
 
 (defun geiser-racket--find-module ()
   (let ((bf (geiser-racket--implicit-module))
@@ -249,10 +249,10 @@ using start-geiser, a procedure in the geiser/server module."
         (mapc 'geiser-edit--buttonize-files geiser-racket--file-rxs)
         (goto-char end)
         (newline))))
-  (or key (not (zerop (length msg)))))
+  (if (and msg (string-match "\\(.+\\)$" msg)) (match-string 1 msg) key))
 
 
-;;; Trying to ascertain whether a buffer is mzscheme scheme:
+;;; Trying to ascertain whether a buffer is racket code:
 
 (defun geiser-racket--guess ()
   (or (save-excursion
@@ -273,7 +273,8 @@ using start-geiser, a procedure in the geiser/server module."
 (defun geiser-racket--keywords ()
   (append geiser-racket-font-lock-forms
           (when geiser-racket-extra-keywords
-            `((,(format "[[(]%s\\>" (regexp-opt geiser-racket-extra-keywords 1))
+            `((,(format "[[(]%s\\>"
+                        (regexp-opt geiser-racket-extra-keywords 1))
                . 1)))))
 
 (geiser-syntax--scheme-indent
@@ -428,6 +429,7 @@ Use a prefix to be asked for a submodule name."
   (import-command geiser-racket--import-command)
   (exit-command geiser-racket--exit-command)
   (find-symbol-begin geiser-racket--symbol-begin)
+  (eval-bounds geiser-racket--eval-bounds)
   (display-error geiser-racket--display-error)
   (external-help geiser-racket--external-help)
   (check-buffer geiser-racket--guess)
